@@ -1,6 +1,10 @@
 /* -- general macros -- */
 
+#include <stdio.h>
+#include <time.h>
+
 #include "config.h"
+#include "abcparse.h"
 
 #define OUTPUTFILE	"Out.ps"	/* standard output file */
 #ifndef WIN32
@@ -36,8 +40,8 @@
 #define STEM_CH_MIN	14	/* min stem height for chords under beams */
 #define STEM_CH_MIN2	10	/* ... for notes with two beams */
 #define STEM_CH_MIN3	 9	/* ... for notes with three beams */
-#define STEM_CH_MIN4	 8	/* ... for notes with four beams */
-#define BEAM_DEPTH	3.2	/* width of a beam stroke (was 2.6) */
+#define STEM_CH_MIN4	 9	/* ... for notes with four beams */
+#define BEAM_DEPTH	3.2	/* width of a beam stroke */
 #define BEAM_OFFSET	0.25	/* pos of flat beam relative to staff line */
 #define BEAM_SHIFT	5.0	/* shift of second and third beams (was 5.3) */
 /*  To align the 4th beam as the 1st: shift=6-(depth-2*offset)/3  */
@@ -68,7 +72,6 @@
 #define MAXSTAFF	16	/* max staves */
 #define BSIZE		512	/* buffer size for one input string */
 #define BUFFSZ		64000	/* size of output buffer */
-#define BUFFSZ1		5000	/* buffer reserved for one staff */
 
 #define BREVE		(BASE_LEN * 2)	/* double note (square note) */
 #define SEMIBREVE	BASE_LEN	/* whole note */
@@ -78,8 +81,6 @@
 #define SEMIQUAVER	(BASE_LEN / 16)	/* 1/16 note */
 
 #define MAXFONTS	30	/* max number of fonts */
-#define MAXENC		6	/* max number of ISO-Latin encodings */
-#define ENC_NATIVE	7	/* (see format.c) */
 
 #define T_LEFT		0
 #define T_JUSTIFY	1
@@ -97,9 +98,11 @@ struct FONTSPEC {
 	float size;
 	float swfac;
 };
-extern char font_enc[MAXFONTS];	/* font encoding */
+extern char *fontnames[MAXFONTS];	/* list of font names */
 
 /* lyrics */
+#define LY_HYPH	0x10	/* replacement character for hyphen */
+#define LY_UNDER 0x11	/* replacement character for underscore */
 #define MAXLY	16	/* max number of lyrics */
 struct lyl {
 	struct FONTSPEC *f;	/* font */
@@ -142,27 +145,32 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define S_EOLN		0x0001		/* end of line */
 #define S_BEAM_ST	0x0002		/* beam starts here */
 #define S_BEAM_BR1	0x0004		/* 2nd beam must restart here */
-#define S_OTHER_HEAD	0x0008		/* don't draw any note head */
-#define S_IN_TUPLET	0x0010		/* in a tuplet */
-#define S_TREM2		0x0020		/* tremolo on 2 notes */
-#define S_RRBAR		0x0040		/* right repeat bar (when bar) */
-#define S_XSTEM		0x0080		/* cross-staff stem (when note) */
-#define S_BEAM_ON	0x0200		/* continue beaming */
-#define S_SL1		0x0400		/* some chord slur start */
-#define S_SL2		0x0800		/* some chord slur end */
-#define S_TI1		0x1000		/* some chord tie start */
-#define S_DYNUP		0x2000		/* dynamic marks above the staff */
-#define S_DYNDN		0x4000		/* dynamic marks below the staff */
+#define S_BEAM_BR2	0x0008		/* 3rd beam must restart here */
+#define S_BEAM_END	0x0010		/* beam ends here */
+#define S_OTHER_HEAD	0x0020		/* don't draw any note head */
+#define S_IN_TUPLET	0x0040		/* in a tuplet */
+#define S_TREM2		0x0080		/* tremolo on 2 notes */
+#define S_RRBAR		0x0100		/* right repeat bar (when bar) */
+#define S_XSTEM		0x0200		/* cross-staff stem (when note) */
+#define S_BEAM_ON	0x0400		/* continue beaming */
+#define S_SL1		0x0800		/* some chord slur start */
+#define S_SL2		0x1000		/* some chord slur end */
+#define S_TI1		0x2000		/* some chord tie start */
 #define S_RBSTOP	0x8000		/* repeat bracket stop */
-#define S_BEAM_BR2	0x00010000	/* 3rd beam must restart here */
 #define S_REPEAT	0x00020000	/* sequence / measure repeat */
 #define S_NL		0x00040000	/* start of new music line */
 #define S_SEQST		0x00080000	/* start of vertical sequence */
 #define S_SECOND	0x00100000	/* symbol on a secondary voice */
 #define S_FLOATING	0x00200000	/* symbol on a floating voice */
 #define S_NOREPBRA	0x00400000	/* don't print the repeat bracket */
-#define S_BEAM_END	0x00800000	/* beam ends here */
-#define S_TREM1		0x01000000	/* tremolo on 1 note */
+#define S_TREM1		0x00800000	/* tremolo on 1 note */
+	unsigned short posit;		/* indication position / staff (2 bits) */
+					/* 0: auto, 1: above, 2: below */
+#define POS_DYN 0			/* shifts */
+#define POS_GCH 2
+#define POS_ORN 4
+#define POS_VOC 6
+#define POS_VOL 8
 	signed char stem;	/* 1 / -1 for stem up / down */
 	signed char nflags;	/* number of note flags when > 0 */
 	char dots;		/* number of dots */
@@ -172,7 +180,7 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 #define H_OVAL		2
 #define H_SQUARE	3
 	signed char multi;	/* multi voice in the staff (+1, 0, -1) */
-	signed char nohdix;	/* no head index */
+	signed char nohdix;	/* no head index (for unison) */
 	unsigned char gcf;	/* font for guitar chords */
 	unsigned char anf;	/* font for annotations */
 	short u;		/* auxillary information:
@@ -183,7 +191,8 @@ struct SYMBOL { 		/* struct for a drawable symbol */
 				 *	- NOTE: tremolo number
 				 *	- FMTCHG (format change): subtype */
 #define PSSEQ 0				/* postscript sequence */
-#define REPEAT 1			/* repeat sequence or measure
+#define SVGSEQ 1			/* SVG sequence */
+#define REPEAT 2			/* repeat sequence or measure
 					 *	doty: # measures if > 0
 					 *	      # notes/rests if < 0
 					 *	nohdix: # repeat */
@@ -227,16 +236,20 @@ struct FORMAT { 		/* struct for page layout */
 	float maxstaffsep, maxsysstaffsep, stemheight;
 	int abc2pscompat, alignbars, aligncomposer, autoclef;
 	int barsperstaff, breakoneoln, bstemdown, comball;
-	int combinevoices, contbarnb, continueall, dynalign;
-	int encoding, exprabove, exprbelow, flatbeams, freegchord;
-	int infoline, gchordbox, graceslurs, gracespace, hyphencont;
-	int landscape, measurebox, measurefirst, measurenb, musiconly;
-	int oneperpage, partsbox, printparts, printtempo;
-	int setdefl, shiftunisson, splittune, squarebreve, staffnonote;
-	int straightflags, stretchstaff, stretchlast;
-	int textoption, titlecaps, titleleft, titletrim, timewarn, tuplets;
-	int vocalabove, withxrefs, writehistory;
-	char *dateformat, *header, *footer, *titleformat;
+	int combinevoices, contbarnb, continueall, dynalign, dynamic;
+	int flatbeams;
+	int infoline, gchord, gchordbox, graceslurs, gracespace, hyphencont;
+	int landscape, linewarn, measurebox, measurefirst, measurenb;
+	int oneperpage, ornament;
+#ifdef HAVE_PANGO
+	int pango;
+#endif
+	int partsbox, pdfmark;
+	int setdefl, shiftunisson, splittune, squarebreve;
+	int staffnonote, straightflags, stretchstaff, stretchlast;
+	int textoption, titlecaps, titleleft, titletrim;
+	int timewarn, transpose, tuplets, vocal, volume;
+	char *bgcolor, *dateformat, *header, *footer, *titleformat;
 #define FONT_UMAX 5		/* max number of user fonts */
 #define ANNOTATIONFONT 5
 #define COMPOSERFONT 6
@@ -261,17 +274,20 @@ struct FORMAT { 		/* struct for page layout */
 	struct FONTSPEC font_tb[FONT_MAX];
 	char ndfont;		/* current index of dynamic fonts */
 	unsigned char gcf, anf, vof;	/* fonts for
-				* guitar chords, annotations and lyrics */
+				 * guitar chords, annotations and lyrics */
+	unsigned short posit;	/* position of indications / staff */
+	unsigned int fields[2];	/* info fields to print
+				 *[0] is 'A'..'Z', [1] is 'a'..'z' */
 };
 
-extern struct FORMAT cfmt;	/* current local format for output */
-extern struct FORMAT dfmt;	/* current global format for output */
+extern struct FORMAT cfmt;	/* current format */
+extern struct FORMAT dfmt;	/* global format */
 
-typedef struct SYMBOL *INFO[26];	/* information fields */
+typedef struct SYMBOL *INFO[26]; /* information fields ('A' .. 'Z') */
 extern INFO info;
 
+extern char *outbuf;		/* output buffer.. should hold one tune */
 extern char *mbf;		/* where to PUTx() */
-extern int nbuf;		/* number of bytes buffered */
 extern int use_buffer;		/* 1 if lines are being accumulated */
 
 extern int outft;		/* last font in the output file */
@@ -286,14 +302,16 @@ extern int defl;		/* decoration flags */
 #define DEF_STEMUP 0x04		/* stem up (1) or down (0) */
 
 		/* switches modified by flags: */
+extern int quiet;		/* quiet mode */
 extern int secure;		/* secure mode */
+extern int annotate;		/* output source references */
 extern int pagenumbers; 	/* write page numbers */
-extern int epsf;		/* for EPSF postscript output */
+extern int epsf;		/* EPSF (1) / SVG (2) output */
+extern int svg;			/* SVG (1) or XML (2 - HTML + SVG) output */
 extern int showerror;		/* show the errors */
 
-extern char outfn[STRL1];	/* output file name */
+extern char outfn[FILENAME_MAX]; /* output file name */
 extern char *in_fname;		/* current input file name */
-extern char *styd;		/* format search directory */
 extern time_t mtime;		/* last modification time of the input file */
 
 extern int file_initialized;	/* for output file */
@@ -337,9 +355,9 @@ extern int nstaff;		/* (0..MAXSTAFF-1) */
 
 struct VOICE_S {
 	struct SYMBOL *sym;	/* associated symbols */
-	struct SYMBOL *last_sym;	/* last symbol while scanning */
+	struct SYMBOL *last_sym; /* last symbol while scanning */
 	struct VOICE_S *next;	/* link */
-	char *name;		/* voice id */
+	char id[VOICE_ID_SZ];	/* voice id */
 	char *nm;		/* voice name */
 	char *snm;		/* voice subname */
 	char *bar_text;		/* bar text at start of staff when bar_start */
@@ -351,6 +369,8 @@ struct VOICE_S {
 	struct clef_s clef;	/* current clef */
 	struct key_s key;	/* current key signature */
 	struct meter_s meter;	/* current time signature */
+	struct key_s ckey;	/* key signature while parsing */
+	struct key_s okey;	/* original key signature while parsing */
 	unsigned hy_st;		/* lyrics hyphens at start of line (bit array) */
 	unsigned ignore:1;	/* ignore this voice */
 	unsigned forced_clef:1;	/* explicit clef */
@@ -362,16 +382,14 @@ struct VOICE_S {
 	unsigned new_name:1;	/* redisplay the voice name */
 	unsigned space:1;	/* have a space before the next note (parsing) */
 	short wmeasure;		/* measure duration while parsing */
+	short transpose;	/* transposition while parsing */
 	short bar_start;	/* bar type at start of staff / 0 */
+	unsigned short posit;	/* position of indications / staff */
 	signed char clone;	/* duplicate from this voice number */
 	unsigned char staff;	/* staff (0..n-1) */
 	unsigned char cstaff;	/* staff while parsing */
-	signed char sfp;	/* key signature while parsing */
 	signed char stem;	/* stem direction while parsing */
 	signed char gstem;	/* grace stem direction while parsing */
-	signed char dyn;	/* place of dynamic marks while parsing */
-	signed char ly_pos;	/* place of lyrics (above / below) */
-	signed char gchord;	/* place of guitar chords (above / below) */
 	unsigned char slur_st;	/* slurs at start of staff */
 };
 extern struct VOICE_S voice_tb[MAXVOICE]; /* voice table */
@@ -390,7 +408,19 @@ struct SYSTEM {			/* staff system */
 	short top_voice;	/* first voice in the staff system */
 	short nstaff;
 	struct {
-		short flags;	/* brace and bracket flags (from %%staves) */
+		short flags;
+#define OPEN_BRACE 0x01
+#define CLOSE_BRACE 0x02
+#define OPEN_BRACKET 0x04
+#define CLOSE_BRACKET 0x08
+#define OPEN_PARENTH 0x10
+#define CLOSE_PARENTH 0x20
+#define STOP_BAR 0x40
+#define FL_VOICE 0x80
+#define OPEN_BRACE2 0x0100
+#define CLOSE_BRACE2 0x0200
+#define OPEN_BRACKET2 0x0400
+#define CLOSE_BRACKET2 0x0800
 		char empty;
 		char dum;
 		struct clef_s clef;
@@ -407,24 +437,27 @@ struct SYSTEM {			/* staff system */
 };
 struct SYSTEM *cursys;		/* current staff system */
 
-/* PUTn: add to buffer with n arguments */
-#define PUT0(f) do {sprintf(mbf,f); a2b(); } while (0)
-#define PUT1(f,a) do {sprintf(mbf,f,a); a2b(); } while (0)
-#define PUT2(f,a,b) do {sprintf(mbf,f,a,b); a2b(); } while (0)
-#define PUT3(f,a,b,c) do {sprintf(mbf,f,a,b,c); a2b(); } while (0)
-#define PUT4(f,a,b,c,d) do {sprintf(mbf,f,a,b,c,d); a2b(); } while (0)
-#define PUT5(f,a,b,c,d,e) do {sprintf(mbf,f,a,b,c,d,e); a2b(); } while (0)
-
 /* -- external routines -- */
 /* abc2ps.c */
 void clrarena(int level);
 int lvlarena(int level);
-char *getarena(int len);
+void *getarena(int len);
 void strext(char *fid, char *ext);
 /* buffer.c */
-void a2b(void);
+#define PUT0(f) a2b(f)
+#define PUT1(f,a) a2b(f,a)
+#define PUT2(f,a,b) a2b(f,a,b)
+#define PUT3(f,a,b,c) a2b(f,a,b,c)
+#define PUT4(f,a,b,c,d) a2b(f,a,b,c,d)
+#define PUT5(f,a,b,c,d,e) a2b(f,a,b,c,d,e)
+void a2b(char *fmt, ...)
+#ifdef __GNUC__
+	__attribute__ ((format (printf, 1, 2)))
+#endif
+	;
 void abskip(float h);
 void buffer_eob(void);
+void marg_init(void);
 void bskip(float h);
 void check_buffer(void);
 void clear_buffer(void);
@@ -433,6 +466,11 @@ void close_page(void);
 float get_bposy(void);
 void write_buffer(void);
 void open_output_file(void);
+int (*output)(FILE *out, const char *fmt, ...)
+#ifdef __GNUC__
+	__attribute__ ((format (printf, 2, 3)))
+#endif
+	;
 void write_eps(void);
 /* deco.c */
 void deco_add(char *text);
@@ -470,7 +508,7 @@ void y_set(struct SYMBOL *s,
 void draw_sym_near(void);
 void draw_all_symb(void);
 float draw_systems(float indent);
-void set_scale(int staff);
+void set_scale(struct SYMBOL *s);
 void output_ps(struct SYMBOL *s, int state);
 void putf(float f);
 void putx(float x);
@@ -478,7 +516,6 @@ void puty(float y);
 void putxy(float x, float y);
 /* format.c */
 void define_fonts(void);
-void define_encodings(void);
 int get_textopt(char *p);
 void interpret_fmt_line(char *w, char *p, int lock);
 void lock_fmt(void *fmt);
@@ -487,7 +524,7 @@ FILE *open_file(char *fn,
 		char *ext,
 		char *rfn);
 void print_format(void);
-int read_fmt_file(char *filename);
+void set_font(int ft);
 void set_format(void);
 struct tblt_s *tblt_parse(char *p);
 /* music.c */
@@ -508,28 +545,46 @@ struct SYMBOL *sym_add(struct VOICE_S *p_voice,
 void bug(char *msg, int fatal);
 void error(int sev, struct SYMBOL *s, char *fmt, ...);
 float scan_u(char *str);
-void add_to_text_block(char *s, int job);
-float cwid(unsigned char c);
-int get_str_font(void);
+float cwid(unsigned short c);
+void get_str_font(int *cft, int *dft);
+void set_str_font(int cft, int dft);
+#ifdef HAVE_PANGO
+void pg_init(void);
+void pg_reset_font(void);
+#endif
 void put_history(void);
 void put_words(struct SYMBOL *words);
-void set_font(int ft);
 void str_font(int ft);
-#define A_LEFT -1
-#define A_CENTER 0
-#define A_RIGHT 1
+#define A_LEFT 0
+#define A_CENTER 1
+#define A_RIGHT 2
+#define A_LYRIC 3
+#define A_GCHORD 4
+#define A_ANNOT 5
+#define A_GCHEXP 6
 void str_out(char *p, int action);
+void str_ft_out(char *p, int end);
 void put_str(char *str, int action);
 float tex_str(char *s);
 extern char tex_buf[];	/* result of tex_str() */
 #define TEX_BUF_SZ 512
-void user_ps_add(char *s);
+char *trim_title(char *p, int first);
+void user_ps_add(char *s, char use);
 void user_ps_write(void);
 void write_title(struct SYMBOL *s);
 void write_heading(struct abctune *t);
 void write_user_ps(void);
-void write_text_block(int job, int abc_state);
+void write_text(char *cmd, char *s, int job);
+/* svg.c */
+void define_svg_symbols(char *title, float w, float h);
+int svg_output(FILE *out, const char *fmt, ...)
+#ifdef __GNUC__
+	__attribute__ ((format (printf, 2, 3)))
+#endif
+	;
+void svg_write(char *buf, int len);
+void svg_close();
 /* syms.c */
-void define_encoding(int enc, char *ename);
+void define_cmap(void);
 void define_font(char *name, int num, int enc);
 void define_symbols(void);
