@@ -3,7 +3,7 @@
  *
  * This file is part of abcm2ps.
  *
- * Copyright (C) 1998-2008 Jean-Franois Moine
+ * Copyright (C) 1998-2012 Jean-François Moine
  * Adapted from abc2ps, Copyright (C) 1996,1997 Michael Methfessel
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,24 +18,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 #include <ctype.h>
 
-#include "abcparse.h"
 #include "abc2ps.h"
 
 struct FORMAT cfmt;		/* current format for output */
 
-char font_enc[MAXFONTS];		/* font encoding */
+char *fontnames[MAXFONTS];		/* list of font names */
+static char font_enc[MAXFONTS];		/* font encoding */
 static char def_font_enc[MAXFONTS];	/* default font encoding */
-static char *fontnames[MAXFONTS];	/* list of font names */
 static char used_font[MAXFONTS];	/* used fonts */
+static float swfac_font[MAXFONTS];	/* width scale */
 static int nfontnames;
 static float staffwidth;
 
@@ -59,6 +57,7 @@ static struct format {
 	{"autoclef", &cfmt.autoclef, FORMAT_B, 0},
 	{"annotationfont", &cfmt.font_tb[ANNOTATIONFONT], FORMAT_F, 0},
 	{"barsperstaff", &cfmt.barsperstaff, FORMAT_I, 0},
+	{"bgcolor", &cfmt.bgcolor, FORMAT_S, 0},
 	{"botmargin", &cfmt.botmargin, FORMAT_U, 0},
 	{"breakoneoln", &cfmt.breakoneoln, FORMAT_B, 0},
 	{"bstemdown", &cfmt.bstemdown, FORMAT_B, 0},
@@ -70,13 +69,11 @@ static struct format {
 	{"continueall", &cfmt.continueall, FORMAT_B, 0},
 	{"dateformat", &cfmt.dateformat, FORMAT_S, 0},
 	{"dynalign", &cfmt.dynalign, FORMAT_B, 0},
-	{"encoding", &cfmt.encoding, FORMAT_I, 1},
-	{"exprabove", &cfmt.exprabove, FORMAT_B, 0},
-	{"exprbelow", &cfmt.exprbelow, FORMAT_B, 0},
+	{"dynamic", &cfmt.dynamic, FORMAT_I, 6},
 	{"footer", &cfmt.footer, FORMAT_S, 0},
 	{"footerfont", &cfmt.font_tb[FOOTERFONT], FORMAT_F, 0},
-	{"freegchord", &cfmt.freegchord, FORMAT_B, 0},
 	{"flatbeams", &cfmt.flatbeams, FORMAT_B, 0},
+	{"gchord", &cfmt.gchord, FORMAT_I, 6},
 	{"gchordbox", &cfmt.gchordbox, FORMAT_B, 0},
 	{"gchordfont", &cfmt.font_tb[GCHORDFONT], FORMAT_F, 3},
 	{"graceslurs", &cfmt.graceslurs, FORMAT_B, 0},
@@ -92,6 +89,7 @@ static struct format {
 	{"landscape", &cfmt.landscape, FORMAT_B, 0},
 	{"leftmargin", &cfmt.leftmargin, FORMAT_U, 0},
 	{"lineskipfac", &cfmt.lineskipfac, FORMAT_R, 0},
+	{"linewarn", &cfmt.linewarn, FORMAT_B, 0},
 	{"maxshrink", &cfmt.maxshrink, FORMAT_R, 0},
 	{"maxstaffsep", &cfmt.maxstaffsep, FORMAT_U, 0},
 	{"maxsysstaffsep", &cfmt.maxsysstaffsep, FORMAT_U, 0},
@@ -99,18 +97,20 @@ static struct format {
 	{"measurefirst", &cfmt.measurefirst, FORMAT_I, 2},
 	{"measurefont", &cfmt.font_tb[MEASUREFONT], FORMAT_F, 2},
 	{"measurenb", &cfmt.measurenb, FORMAT_I, 0},
-	{"musiconly", &cfmt.musiconly, FORMAT_B, 0},
 	{"musicspace", &cfmt.musicspace, FORMAT_U, 0},
 	{"notespacingfactor", &cfmt.notespacingfactor, FORMAT_R, 1},
 	{"oneperpage", &cfmt.oneperpage, FORMAT_B, 0},
+	{"ornament", &cfmt.ornament, FORMAT_I, 6},
 	{"pageheight", &cfmt.pageheight, FORMAT_U, 0},
 	{"pagewidth", &cfmt.pagewidth, FORMAT_U, 0},
+#ifdef HAVE_PANGO
+	{"pango", &cfmt.pango, FORMAT_B, 2},
+#endif
 	{"parskipfac", &cfmt.parskipfac, FORMAT_R, 0},
 	{"partsbox", &cfmt.partsbox, FORMAT_B, 0},
 	{"partsfont", &cfmt.font_tb[PARTSFONT], FORMAT_F, 1},
 	{"partsspace", &cfmt.partsspace, FORMAT_U, 0},
-	{"printparts", &cfmt.printparts, FORMAT_B, 0},
-	{"printtempo", &cfmt.printtempo, FORMAT_B, 0},
+	{"pdfmark", &cfmt.pdfmark, FORMAT_I, 0},
 	{"repeatfont", &cfmt.font_tb[REPEATFONT], FORMAT_F, 0},
 	{"rightmargin", &cfmt.rightmargin, FORMAT_U, 0},
 	{"scale", &cfmt.scale, FORMAT_R, 0},
@@ -150,32 +150,19 @@ static struct format {
 	{"timewarn", &cfmt.timewarn, FORMAT_B, 0},
 	{"topmargin", &cfmt.topmargin, FORMAT_U, 0},
 	{"topspace", &cfmt.topspace, FORMAT_U, 0},
+	{"transpose", &cfmt.transpose, FORMAT_I, 1},
 	{"tuplets", &cfmt.tuplets, FORMAT_I, 3},
-	{"vocalabove", &cfmt.vocalabove, FORMAT_B, 0},
+	{"vocal", &cfmt.vocal, FORMAT_I, 6},
 	{"vocalfont", &cfmt.font_tb[VOCALFONT], FORMAT_F, 0},
 	{"vocalspace", &cfmt.vocalspace, FORMAT_U, 0},
 	{"voicefont", &cfmt.font_tb[VOICEFONT], FORMAT_F, 0},
-	{"withxrefs", &cfmt.withxrefs, FORMAT_B, 0},
+	{"volume", &cfmt.volume, FORMAT_I, 6},
 	{"wordsfont", &cfmt.font_tb[WORDSFONT], FORMAT_F, 0},
 	{"wordsspace", &cfmt.wordsspace, FORMAT_U, 0},
-	{"writehistory", &cfmt.writehistory, FORMAT_B, 0},
+	{"writefields", &cfmt.fields, FORMAT_B, 1},
 	{0, 0, 0, 0}		/* end of table */
 };
 
-static char *enc_name[20] = {
-	"us-ascii", 	/* 0 */
-	"iso-8859-1", 	/* 1 */
-	"iso-8859-2", 	/* 2 */
-	"iso-8859-3", 	/* 3 */
-	"iso-8859-4", 	/* 4 */
-	"iso-8859-9",	/* 5 (Latin5) */
-	"iso-8859-10",	/* 6 (Latin6) */
-	"native",	/* 7 = ENC_NATIVE */
-#if 0
-	"utf-8",	/* 8 */
-#endif
-};
-
 /* -- search a font and add it if not yet defined -- */
 static int get_font(char *fname, int encoding)
 {
@@ -188,24 +175,27 @@ static int get_font(char *fname, int encoding)
 				encoding = def_font_enc[fnum];
 			if (encoding == font_enc[fnum])
 				return fnum;		/* font found */
+			break;
 		}
-	for ( ; --fnum >= 0; )
+	while (--fnum >= 0) {
 		if (strcmp(fname, fontnames[fnum]) == 0
-		    && encoding == font_enc[fnum])
+		 && encoding == font_enc[fnum])
 			return fnum;
+	}
 
 	/* add the font */
 	if (nfontnames >= MAXFONTS) {
 		error(1, 0, "Too many fonts");
 		return 0;
 	}
-	if (file_initialized)
+	if (file_initialized
+	 && (epsf != 2 && !svg))
 		error(1, 0,
 		      "Cannot have a new font when the output file is opened");
 	fnum = nfontnames++;
 	fontnames[fnum] = strdup(fname);
 	if (encoding < 0)
-		encoding = cfmt.encoding;
+		encoding = 0;
 	font_enc[fnum] = encoding;
 	return fnum;
 }
@@ -240,17 +230,21 @@ static void fontspec(struct FONTSPEC *f,
 	else	name = fontnames[f->fnum];
 	f->size = size;
 	f->swfac = size;
-	if (strncmp(name, "Times", 5) == 0) {
+	if (swfac_font[f->fnum] != 0) {
+		f->swfac *= swfac_font[f->fnum];
+	} else if (strncmp(name, "Times", 5) == 0) {
 		if (strcmp(name, "Times-Bold") == 0)
 			f->swfac *= 1.05;
-	} else if (strcmp(name, "Helvetica-Bold") == 0)
+	} else if (strcmp(name, "Helvetica-Bold") == 0) {
 		f->swfac *= 1.15;
-	else if (strncmp(name, "Helvetica", 9) == 0
-		 || strncmp(name, "Palatino", 8) == 0)
+	} else if (strncmp(name, "Helvetica", 9) == 0
+		|| strncmp(name, "Palatino", 8) == 0) {
 		f->swfac *= 1.10;
-	else if (strncmp(name, "Courier", 7) == 0)
+	} else if (strncmp(name, "Courier", 7) == 0) {
 		f->swfac *= 1.35;
-	else	f->swfac *= 1.2;		/* unknown font */
+	} else {
+		f->swfac *= 1.2;		/* unknown font */
+	}
 	if (f == &cfmt.font_tb[GCHORDFONT])
 		cfmt.gcf = dfont_set(f);
 	else if (f == &cfmt.font_tb[ANNOTATIONFONT])
@@ -259,30 +253,12 @@ static void fontspec(struct FONTSPEC *f,
 		cfmt.vof = dfont_set(f);
 }
 
-/* -- output the encodings -- */
-void define_encodings(void)
-{
-	int i, enc;
-
-	enc = 0;
-	for (i = 0; i < nfontnames; i++) {
-		if (used_font[i])
-			enc |= 1 << font_enc[i];
-	}
-	for (i = 0; ; i++) {
-		if (enc == 0)
-			break;
-		if (enc & 1)
-			define_encoding(i, enc_name[i]);
-		enc >>= 1;
-	}
-}
-
 /* -- output the font definitions with their encodings -- */
 void define_fonts(void)
 {
 	int i;
 
+	define_cmap();
 	for (i = 0; i < nfontnames; i++) {
 		if (used_font[i])
 			define_font(fontnames[i], i, font_enc[i]);
@@ -356,6 +332,7 @@ static void set_infoname(char *p)
 }
 
 /* -- set the default format -- */
+/* this function is called only once, at abcm2ps startup time */
 void set_format(void)
 {
 	struct FORMAT *f;
@@ -392,17 +369,22 @@ void set_format(void)
 	f->autoclef = 1;
 	f->breakoneoln = 1;
 	f->dynalign = 1;
-	f->printparts = 1;
-	f->printtempo = 1;
+	f->linewarn = 1;
+#ifdef HAVE_PANGO
+	if (!svg && epsf != 2)
+		f->pango = 1;
+	else
+		lock_fmt(&cfmt.pango);	/* SVG output does not use panga */
+#endif
 	f->staffnonote = 1;
 	f->titletrim = 1;
 	f->aligncomposer = A_RIGHT;
 	f->notespacingfactor = 1.414;
 	f->stemheight = STEM;
 #ifndef WIN32
-	f->dateformat = strdup("\\%b \\%e, \\%Y \\%H:\\%M");
+	f->dateformat = strdup("%b %e, %Y %H:%M");
 #else
-	f->dateformat = strdup("\\%b \\%#d, \\%Y \\%H:\\%M");
+	f->dateformat = strdup("%b %#d, %Y %H:%M");
 #endif
 	f->gracespace = (65 << 16) | (80 << 8) | 120;	/* left-inside-right - unit 1/10 pt */
 	f->textoption = T_LEFT;
@@ -424,6 +406,13 @@ void set_format(void)
 	fontspec(&f->font_tb[VOCALFONT], "Times-Bold", 0, 13.0);
 	fontspec(&f->font_tb[VOICEFONT], "Times-Bold", 0, 13.0);
 	fontspec(&f->font_tb[WORDSFONT], "Times-Roman", 0, 16.0);
+	f->fields[0] = (1 << ('C' - 'A'))
+		| (1 << ('O' - 'A'))
+		| (1 << ('P' - 'A'))
+		| (1 << ('Q' - 'A'))
+		| (1 << ('T' - 'A'))
+		| (1 << ('W' - 'A'));
+	f->fields[1] = (1 << ('w' - 'a'));
 	set_infoname("R \"Rhythm: \"");
 	set_infoname("B \"Book: \"");
 	set_infoname("S \"Source: \"");
@@ -437,19 +426,62 @@ void set_format(void)
 void print_format(void)
 {
 	struct format *fd;
-static char yn[2][5]={"no","yes"};
+static char yn[2][5] = {"no","yes"};
+static char posit[3][8] = {"auto", "above", "below"};
 
 	for (fd = format_tb; fd->name; fd++) {
-		printf("  %-13s ", fd->name);
+		printf("%-15s ", fd->name);
 		switch (fd->type) {
+		case FORMAT_B:
+			switch (fd->subtype) {
+#ifdef HAVE_PANGO
+			case 2:				/* pango = 0, 1 or 2 */
+				if (cfmt.pango == 2) {
+					printf("2\n");
+					break;
+				}
+				/* fall thru */
+#endif
+			case 0:
+				printf("%s\n", yn[*((int *) fd->v)]);
+				break;
+			case 1: {			/* writefields */
+				int i;
+
+				for (i = 0; i < 32; i++) {
+					if (cfmt.fields[0] & (1 << i))
+						printf("%c", (char) ('A' + i));
+					if (cfmt.fields[1] & (1 << i))
+						printf("%c", (char) ('a' + i));
+				}
+				printf("\n");
+				break;
+			    }
+			}
+			break;
 		case FORMAT_I:
 			switch (fd->subtype) {
 			default:
 				printf("%d\n", *((int *) fd->v));
 				break;
-			case 1:			/* encoding */
-				printf("%s\n", enc_name[*((int *) fd->v)]);
+			case 1: {		/* transpose */
+				int t;
+
+				t = *((int *) fd->v);
+				if (t >= 0)
+					putchar('+');
+				printf("%d", t / 3);
+				switch ((t + 240) % 3) {
+				case 1:
+					putchar('#');
+					break;
+				case 2:
+					putchar('b');
+					break;
+				}
+				putchar('\n');
 				break;
+			    }
 			case 3:			/* tuplets */
 				printf("%d %d %d\n",
 					cfmt.tuplets >> 8,
@@ -465,6 +497,9 @@ static char yn[2][5]={"no","yes"};
 					(cfmt.gracespace & 0xff) / 10,
 					(cfmt.gracespace & 0xff) % 10);
 				break;
+			case 6:			/* position */
+				printf("%s\n", posit[*((int *) fd->v)]);
+				break;
 			}
 			break;
 		case FORMAT_R:
@@ -475,13 +510,13 @@ static char yn[2][5]={"no","yes"};
 
 			s = (struct FONTSPEC *) fd->v;
 			printf("%s", fontnames[s->fnum]);
-			if (font_enc[s->fnum] != cfmt.encoding)
-				printf(" %s",
-				       enc_name[(unsigned) font_enc[s->fnum]]);
+//			if (font_enc[s->fnum] != cfmt.encoding)
+				printf(" %s", font_enc[s->fnum] ?
+						"native" : "utf-8");
 			printf(" %.1f", s->size);
 			if ((fd->subtype == 1 && cfmt.partsbox)
-			    || (fd->subtype == 2 && cfmt.measurebox)
-			    || (fd->subtype == 3 && cfmt.gchordbox))
+			 || (fd->subtype == 2 && cfmt.measurebox)
+			 || (fd->subtype == 3 && cfmt.gchordbox))
 				printf(" box");
 			printf("\n");
 			break;
@@ -489,17 +524,16 @@ static char yn[2][5]={"no","yes"};
 		case FORMAT_U:
 			if (fd->subtype == 0)
 				printf("%.2fcm\n", *((float *) fd->v) / (1 CM));
-			else	printf("%.2fcm\n",
+			else
+				printf("%.2fcm\n",
 					(cfmt.pagewidth
 						- cfmt.leftmargin
 						- cfmt.rightmargin)
 					/ (1 CM));
 			break;
-		case FORMAT_B:
-			printf("%s\n", yn[*((int *) fd->v)]);
-			break;
 		case FORMAT_S:
-			printf("\"%s\"\n", *((char **) fd->v) != 0 ? *((char **) fd->v) : "");
+			printf("\"%s\"\n",
+				*((char **) fd->v) != 0 ? *((char **) fd->v) : "");
 			break;
 		}
 	}
@@ -509,62 +543,54 @@ static char yn[2][5]={"no","yes"};
 static int get_encoding(char *p)
 {
 	int l;
-	char **e;
 
 	l = 0;
 	while (p[l] != '\0' && !isspace((unsigned char) p[l]))
 		l++;
-	if (strncasecmp(p, "ASCII", l) == 0)	/* backward compatibility */
-		return 0;
-	e = enc_name;
-	for (;;) {
-		if (strncmp(p, *e, l) == 0)
-			break;
-		e++;
-		if (*e == 0
-		    || e >= &enc_name[sizeof enc_name]) {
-			if (e >= &enc_name[sizeof enc_name - 1]) {
-				error(1, 0, "Too many encodings");
-				return 0;
-			}
-			*e = (char *) malloc(l + 1);
-			strncpy(*e, p, l);
-			(*e)[l] = '\0';
-		}
-	}
-	return e - enc_name;
+	if (strncasecmp(p, "native", l) == 0)
+		return 1;
+	/* no error */
+	return 0;
+}
+
+/* -- get a position -- */
+static int get_posit(char *p)
+{
+	if (strcmp(p, "up") == 0
+	 || strcmp(p, "above") == 0)
+		return SL_ABOVE;
+	if (strcmp(p, "down") == 0
+	 || strcmp(p, "below") == 0)
+		return SL_BELOW;
+	return 0;			/* auto */
 }
 
 /* -- get the option for text -- */
 int get_textopt(char *p)
 {
-	int option;
-
-	option = T_LEFT;
 	if (*p == '\0'
-	    || strncmp(p, "obeylines", 9) == 0)
-		;
-	else if (strncmp(p, "align", 5) == 0
-		 || strncmp(p, "justify", 7) == 0)
-		option = T_JUSTIFY;
-	else if (strncmp(p, "ragged", 6) == 0
-		 || strncmp(p, "fill", 4) == 0)
-		option = T_FILL;
-	else if (strncmp(p, "center", 6) == 0)
-		option = T_CENTER;
-	else if (strncmp(p, "skip", 4) == 0)
-		option = T_SKIP;
-	else if (strncmp(p, "right", 5) == 0)
-		option = T_RIGHT;
-	else	option = -1;
-	return option;
+	 || strncmp(p, "obeylines", 9) == 0)
+		return T_LEFT;
+	if (strncmp(p, "align", 5) == 0
+	 || strncmp(p, "justify", 7) == 0)
+		return T_JUSTIFY;
+	if (strncmp(p, "ragged", 6) == 0
+	 || strncmp(p, "fill", 4) == 0)
+		return T_FILL;
+	if (strncmp(p, "center", 6) == 0)
+		return T_CENTER;
+	if (strncmp(p, "skip", 4) == 0)
+		return T_SKIP;
+	if (strncmp(p, "right", 5) == 0)
+		return T_RIGHT;
+	return -1;
 }
 
 /* -- get a boolean value -- */
 static int g_logv(char *p)
 {
 	switch (*p) {
-	case 0:
+	case '\0':
 	case '1':
 	case 'y':
 	case 'Y':
@@ -578,19 +604,12 @@ static int g_logv(char *p)
 	case 'F':
 		break;
 	default:
-		fprintf(stderr,
-			"++++ Unknown logical '%s' - false assumed\n", p);
+		error(0, 0, "Unknown logical '%s' - false assumed", p);
 		break;
 	}
 	return 0;
 }
 
-/* --  get a float variable, no units -- */
-static float g_fltv(char *p)
-{
-	return atof(p);
-}
-
 /* -- get a font specifier -- */
 static void g_fspc(char *p,
 		   struct FONTSPEC *f)
@@ -600,16 +619,29 @@ static void g_fspc(char *p,
 	float fsize;
 
 	p = get_str(fname, p, sizeof fname);
-	if (!isdigit((unsigned char) *p)) {
+	if (isalpha((unsigned char) *p) || *p == '*') {
 		if (*p == '*')
 			encoding = font_enc[f->fnum];
-		else	encoding = get_encoding(p);
+		else
+			encoding = get_encoding(p);
 		while (*p != '\0' && !isspace((unsigned char) *p))
 			p++;
 		while (isspace((unsigned char) *p))
 			p++;
-	} else	encoding = -1;
-	fsize = *p != '\0' && *p != '*' ? g_fltv(p) : f->size;
+	} else {
+		encoding = -1;
+	}
+	fsize = f->size;
+	if (*p != '\0' && *p != '*') {
+		char *q;
+		float v;
+
+		v = strtod(p, &q);
+		if (v <= 0 || (*q != '\0' && *q != ' '))
+			error(1, 0, "Bad font size '%s'", p);
+		else
+			fsize = v;
+	}
 	fontspec(f,
 		 strcmp(fname, "*") != 0 ? fname : 0,
 		 encoding,
@@ -618,6 +650,9 @@ static void g_fspc(char *p,
 		used_font[f->fnum] = 1;
 	if (f - cfmt.font_tb == outft)
 		outft = -1;
+#ifdef HAVE_PANGO
+	pg_reset_font();
+#endif
 }
 
 /* -- parse a 'tablature' definition -- */
@@ -645,7 +680,7 @@ struct tblt_s *tblt_parse(char *p)
 		p++;
 		n = *p++ - '0' - 1;
 		if ((unsigned) n >= MAXTBLT
-		    || (*p != '\0' && *p != ' ')) {
+		 || (*p != '\0' && *p != ' ')) {
 			error(1, 0, "Invalid number in %%%%tablature");
 			return 0;
 		}
@@ -679,18 +714,21 @@ struct tblt_s *tblt_parse(char *p)
 		while (*p == '\'' || *p == ',') {
 			if (*p++ == '\'')
 				tblt->pitch += 12;
-			else	tblt->pitch -= 12;
+			else
+				tblt->pitch -= 12;
 		}
 		if (*p == '#' || *p == 'b') {
 			if (*p == '#')
 				tblt->pitch++;
-			else	tblt->pitch--;
+			else
+				tblt->pitch--;
 			tblt->instr[1] = *p++;
 		}
 		while (*p == '\'' || *p == ',') {
 			if (*p++ == '\'')
 				tblt->pitch += 12;
-			else	tblt->pitch -= 12;
+			else
+				tblt->pitch -= 12;
 		}
 		while (isspace((unsigned char) *p))
 			p++;
@@ -777,27 +815,61 @@ void interpret_fmt_line(char *w,		/* keyword */
 			return;
 		}
 		break;
+	case 'e':
+		if (strcmp(w, "exprabove") == 0) {	/* compatibility */
+			cfmt.posit &= ~((1 << POS_DYN) | (1 << POS_VOL));
+			if (g_logv(p))
+				cfmt.posit |= (1 << POS_DYN) | (1 << POS_VOL);
+			return;
+		}
+		if (strcmp(w, "exprbelow") == 0) {	/* compatibility */
+			cfmt.posit &= ~((1 << POS_DYN) | (1 << POS_VOL));
+			if (g_logv(p))
+				cfmt.posit |= (2 << POS_DYN) | (2 << POS_VOL);
+			return;
+		}
+		break;
 	case 'f':
 		if (strcmp(w, "font") == 0) {
 			int fnum, encoding;
+			float swfac;
 			char fname[80];
 
+			if (file_initialized) {
+				error(1, 0,
+				      "Cannot define a font when the output file is opened");
+				return;
+			}
 			p = get_str(fname, p, sizeof fname);
-			if (*p == '\0')
-				encoding = cfmt.encoding;
-			else	encoding = get_encoding(p);
+			swfac = 0;			/* defaults to 1.2 */
+			encoding = 0;
+			if (*p != '\0') {
+				if (isalpha((unsigned char) *p)) {
+					encoding = get_encoding(p);
+					while (*p != '\0'
+					    && !isspace((unsigned char) *p))
+						p++;
+					while (isspace((unsigned char) *p))
+						p++;
+				}
+				if (isdigit((unsigned char) *p)) {
+					char *q;
+					float v;
+
+					v = strtod(p, &q);
+					if (v > 2 || (*q != '\0' && *q != '\0')) {
+						error(1, 0, "Bad value '%s' for '%s'", p, w);
+						break;
+					}
+					swfac = v;
+				}
+			}
 			fnum = get_font(fname, encoding);
 			def_font_enc[fnum] = encoding;
+			swfac_font[fnum] = swfac;
 			used_font[fnum] = 1;
 			return;
 		}
-		if (strcmp(w, "format") == 0) {
-			if (secure)
-				return;
-			if (read_fmt_file(p) < 0)
-				error(1, 0, "No such format file '%s'", p);
-			return;
-		}
 		break;
 	case 'i':
 		if (strcmp(w, "infoname") == 0) {
@@ -810,12 +882,35 @@ void interpret_fmt_line(char *w,		/* keyword */
 			return;
 		}
 		break;
+	case 'm':
+		if (strcmp(w, "musiconly") == 0) {	/* compatibility */
+			if (g_logv(p))
+				cfmt.fields[1] &= ~(1 << ('w' - 'a'));
+			else
+				cfmt.fields[1] |= (1 << ('w' - 'a'));
+			return;
+		}
+		break;
 	case 'p':
 		if (strcmp(w, "postscript") == 0) {
-			if (!file_initialized)
-				user_ps_add(p);
+			if (!file_initialized && mbf == outbuf)
+				user_ps_add(p, 'b');
+			else if (svg || epsf == 2 || !secure)
+				a2b("%s\n", p);
+			return;
+		}
+		if (strcmp(w, "printparts") == 0) {	/* compatibility */
+			if (g_logv(p))
+				cfmt.fields[0] |= (1 << ('P' - 'A'));
 			else
-				PUT1("%s\n", p);
+				cfmt.fields[0] &= ~(1 << ('P' - 'A'));
+			return;
+		}
+		if (strcmp(w, "printtempo") == 0) {	/* compatibility */
+			if (g_logv(p))
+				cfmt.fields[0] |= (1 << ('Q' - 'A'));
+			else
+				cfmt.fields[0] &= ~(1 << ('Q' - 'A'));
 			return;
 		}
 		break;
@@ -825,22 +920,111 @@ void interpret_fmt_line(char *w,		/* keyword */
 			return;
 		}
 		break;
+	case 'v':
+		if (strcmp(w, "vocalabove") == 0) {	/* compatibility */
+			cfmt.posit &= ~(1 << POS_VOC);
+			if (g_logv(p))
+				cfmt.posit |= (1 << POS_VOC);
+			return;
+		}
+		break;
+	case 'w':
+		if (strcmp(w, "withxrefs") == 0) {	/* compatibility */
+			if (g_logv(p))
+				cfmt.fields[0] |= (1 << ('X' - 'A'));
+			else
+				cfmt.fields[0] &= ~(1 << ('X' - 'A'));
+			return;
+		}
+		if (strcmp(w, "writehistory") == 0) {	/* compatibility */
+			struct SYMBOL *s;
+			int bool;
+			unsigned u;
+
+			bool = g_logv(p);
+			for (s = info['I' - 'A']; s != 0; s = s->next) {
+				u = s->as.text[0] - 'A';
+				if (bool)
+					cfmt.fields[0] |= (1 << u);
+				else
+					cfmt.fields[0] &= ~(1 << u);
+			}
+			return;
+		}
+		break;
 	}
 	for (fd = format_tb; fd->name; fd++)
 		if (strcmp(w, fd->name) == 0)
 			break;
 	if (fd->name == 0)
 		return;
-	if (fd->lock && !lock)
+
+	{
+		int l;
+
+		l = strlen(p);
+		if (strcmp(p + l - 5, " lock") == 0) {
+			p[l - 5] = '\0';
+			lock = 1;
+		}
+	}
+	if (lock)
+		fd->lock = 1;
+	else if (fd->lock)
 		return;
-	fd->lock |= lock;
+
 	switch (fd->type) {
+	case FORMAT_B:
+		switch (fd->subtype) {
+#ifdef HAVE_PANGO
+		case 2:				/* %%pango = 0, 1 or 2 */
+			if (*p == '2') {
+				cfmt.pango = 2;
+				break;
+			}
+			/* fall thru */
+#endif
+		case 0:
+			*((int *) fd->v) = g_logv(p);
+			break;
+		case 1:	{			/* %%writefields */
+			char *q;
+			int bool, i;
+			unsigned u;
+
+			q = p;
+			while (*p != '\0' && !isspace((unsigned char) *p))
+				p++;
+			while (isspace((unsigned char) *p))
+				p++;
+			bool = g_logv(p);
+			while (*q != '\0' && !isspace((unsigned char) *q)) {
+				u = *q - 'A';
+				if (u < 26) {
+					i = 0;
+				} else {
+					u = *q - 'a';
+					if (u < 26)
+						i = 1;
+					else
+						break;	/*fixme: error */
+				}
+				if (bool)
+					cfmt.fields[i] |= (1 << u);
+				else
+					cfmt.fields[i] &= ~(1 << u);
+				q++;
+			}
+			break;
+		    }
+		}
+		break;
 	case FORMAT_I:
 		if (fd->subtype == 3) {		/* tuplets */
 			unsigned i1, i2, i3;
 
 			if (sscanf(p, "%d %d %d", &i1, &i2, &i3) != 3
-			    || i1 > 2 || i2 > 2 || i3 > 2) {
+			 || i1 > 2 || i2 > 2 || i3 > 2) {
 				error(1, 0,
 				      "Bad 'tuplets' values '%s' - ignored",
 				      p);
@@ -854,7 +1038,7 @@ void interpret_fmt_line(char *w,		/* keyword */
 			float f1, f2, f3;
 
 			if (sscanf(p, "%f %f %f", &f1, &f2, &f3) != 3
-			    || f1 > 256 || f2 > 256 || f3 > 256) {
+			 || f1 > 256 || f2 > 256 || f3 > 256) {
 				error(1, 0,
 				      "Bad 'gracespace' values '%s' - ignored",
 				      p);
@@ -866,27 +1050,25 @@ void interpret_fmt_line(char *w,		/* keyword */
 			cfmt.gracespace = (i1 << 16) | (i2 << 8) | i3;
 			break;
 		}
-		if (fd->subtype == 1 && !isdigit(*p))	/* 'encoding' */
-			cfmt.encoding = get_encoding(p);
-		else if (fd->subtype == 4 && !isdigit(*p)) /* 'textoption' */
+		if (fd->subtype == 4 && !isdigit(*p)) /* 'textoption' */
 			cfmt.textoption = get_textopt(p);
-		else	sscanf(p, "%d", (int *) fd->v);
+		else if (fd->subtype == 6 && !isdigit(*p)) /* position */
+			*((int *) fd->v) = get_posit(p);
+		else
+			sscanf(p, "%d", (int *) fd->v);
 		switch (fd->subtype) {
-		case 1:					/* 'encoding' */
-			if (isdigit(*p)
-			    && (unsigned) cfmt.encoding > MAXENC) {
-				error(1, 0,
-				      "Bad encoding value %d - reset to 0",
-				      cfmt.encoding);
-				cfmt.encoding = 0;
-			} else {
-				int i;
-
-				for (i = 0; i < nfontnames; i++) {
-					if (font_enc[i] == 0)
-						def_font_enc[i] = font_enc[i]
-							= cfmt.encoding;
-				}
+		case 1:					/* 'transpose' */
+			cfmt.transpose *= 3;
+			if (p[strlen(p) - 1] == '#') {
+				if (cfmt.transpose > 0)
+					cfmt.transpose++;
+				else
+					cfmt.transpose -= 2;
+			} else if (p[strlen(p) - 1] == 'b') {
+				if (cfmt.transpose > 0)
+					cfmt.transpose += 2;
+				else
+					cfmt.transpose--;
 			}
 			break;
 		case 2:
@@ -900,31 +1082,54 @@ void interpret_fmt_line(char *w,		/* keyword */
 				cfmt.textoption = T_LEFT;
 			}
 			break;
+		case 6:				/* position */
+			if ((unsigned) *((int *) fd->v) > 2) {
+				error(1, 0,
+				      "Bad position - set to 'auto'",
+				      *((int *) fd->v));
+				*((int *) fd->v) = SL_AUTO;
+			}
+			cfmt.posit = (cfmt.dynamic << POS_DYN)
+				| (cfmt.gchord << POS_GCH)
+				| (cfmt.ornament << POS_ORN)
+				| (cfmt.vocal << POS_VOC)
+				| (cfmt.volume << POS_VOL);
+			break;
 		}
 		break;
 	case FORMAT_R:
-		*((float *) fd->v) = g_fltv(p);
-		if (fd->subtype == 1) {	/* note spacing factor */
+		{
+			char *q;
+			float v;
+
+			v = strtod(p, &q);
+			if (v <= 0 || (*q != '\0' && *q != ' ')) {
+				error(1, 0, "Bad value '%s' for '%s'", p, w);
+				break;
+			}
+			*((float *) fd->v) = v;
+		}
+		if (fd->subtype == 1) {		/* note spacing factor */
 			int i;
-			float w;
+			float v;
 
 			if (cfmt.notespacingfactor <= 0) {
-				fprintf(stderr,
-					"Bad value for 'notespacingfactor'\n");
+				error(1, 0,
+					"Bad value for 'notespacingfactor'");
 				cfmt.notespacingfactor = 1;
 				break;
 			}
 			i = C_XFLAGS;		/* crotchet index */
-			w = space_tb[i];
+			v = space_tb[i];
 			for ( ; --i >= 0; ) {
-				w /= cfmt.notespacingfactor;
-				space_tb[i] = w;
+				v /= cfmt.notespacingfactor;
+				space_tb[i] = v;
 			}
 			i = C_XFLAGS;
-			w = space_tb[i];
+			v = space_tb[i];
 			for ( ; ++i < NFLAGS_SZ; ) {
-				w *= cfmt.notespacingfactor;
-				space_tb[i] = w;
+				v *= cfmt.notespacingfactor;
+				space_tb[i] = v;
 			}
 		}
 		break;
@@ -952,15 +1157,12 @@ void interpret_fmt_line(char *w,		/* keyword */
 			float rmargin;
 
 			rmargin = (cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
-				- staffwidth - cfmt.leftmargin;
+					- staffwidth - cfmt.leftmargin;
 			if (rmargin < 0)
-				fprintf(stderr, "'staffwidth' too big\n");
+				error(1, 0, "'staffwidth' too big\n");
 			cfmt.rightmargin = rmargin;
 		}
 		break;
-	case FORMAT_B:
-		*((int *) fd->v) = g_logv(p);
-		break;
 	case FORMAT_S: {
 		int l;
 
@@ -968,43 +1170,13 @@ void interpret_fmt_line(char *w,		/* keyword */
 		*((char **) fd->v) = getarena(l);
 		if (*p == '"')
 			get_str(*((char **) fd->v), p, l);
-		else	strcpy(*((char **) fd->v), p);
+		else
+			strcpy(*((char **) fd->v), p);
 		break;
 	    }
 	}
 }
 
-/* -- remove the spaces and comments of a format line -- */
-static char *clean_line(char *p)
-{
-	int i;
-	char c, *q;
-
-	while (isspace((unsigned char) *p))
-		p++;
-	q = p;
-	i = 0;
-	for (;;) {
-		if ((c = *p++) == '%') {
-			if (i > 0 && p[-2] != '\\')
-				c = '\0';
-		}
-		if (c == '\0') {
-			p--;
-			break;
-		}
-		i++;
-	}
-	while (--i > 0) {
-		c = *--p;
-		if (!isspace((unsigned char) c)) {
-			p[1] = '\0';
-			break;
-		}
-	}
-	return q;
-}
-
 /* -- lock a format -- */
 void lock_fmt(void *fmt)
 {
@@ -1018,125 +1190,6 @@ void lock_fmt(void *fmt)
 	fd->lock = 1;
 }
 
-/* -- open a file for reading -- */
-FILE *open_file(char *fn,	/* file name */
-		char *ext,	/* file type */
-		char *rfn)	/* real file name */
-{
-	FILE *fp;
-	char *p;
-	int l;
-
-	strcpy(rfn, fn);
-	if ((fp = fopen(rfn, "r")) != 0)
-		return fp;
-	strext(rfn, ext);
-	if ((fp = fopen(rfn, "r")) != 0)
-		return fp;
-	if (in_fname != 0
-	    && (p = strrchr(in_fname, DIRSEP)) != 0) {
-		l = p - in_fname + 1;
-		strncpy(rfn, in_fname, l);
-		strcpy(&rfn[l], fn);
-		if ((fp = fopen(rfn, "r")) != 0)
-			return fp;
-		strext(&rfn[l], ext);
-		if ((fp = fopen(rfn, "r")) != 0)
-			return fp;
-	}
-	if (*styd == '\0')
-		return 0;
-	sprintf(rfn, "%s%c%s", styd, DIRSEP, fn);
-	if ((fp = fopen(rfn, "r")) != 0)
-		return fp;
-	strext(rfn, ext);
-	if ((fp = fopen(rfn, "r")) != 0)
-		return fp;
-	return 0;
-}
-
-/* -- read a format file -- */
-int read_fmt_file(char *fn)
-{
-	FILE *fp;
-	char line[BSIZE], *p, *q;
-	static int nbfiles;
-
-	if (nbfiles > 3) {
-		error(1, 0, "Too many simultaneous files");
-		return 0;
-	}
-	line[0] = '\001';
-	if ((fp = open_file(fn, "fmt", &line[1])) == 0)
-		return -1;
-	if (strcmp(&line[strlen(line) - 3], ".ps") == 0) {
-		if (!file_initialized)
-			user_ps_add(line);
-		else
-			error(0, 0, "Cannot include the file %s", &line[1]);
-		fclose(fp);
-		return 0;
-	}
-	nbfiles++;
-	for (;;) {
-		p = line;
-		if (!fgets(p, sizeof line, fp))
-			break;
-		p = clean_line(p);
-		if (*p == '\0')
-			continue;
-		if (strcmp(p, "beginps") == 0) {
-			for (;;) {
-				p = line;
-				if (!fgets(p, sizeof line, fp))
-					break;
-#if 1
-				{
-					int i;
-					char c;
-
-					i = strlen(p);
-					while (--i > 0) {
-						c = p[i];
-						if (!isspace((unsigned char) c)) {
-							p[i + 1] = '\0';
-							break;
-						}
-					}
-				}
-#else
-				p = clean_line(p);
-#endif
-				if (*p == '\0')
-					continue;
-				if (strcmp(p, "endps") == 0)
-					break;
-				if (!secure) {
-					if (!file_initialized)
-						user_ps_add(p);
-					else
-						PUT1("%s\n", p);
-				}
-			}
-			continue;
-		}
-		if (strcmp(p, "end") == 0)
-			break;
-		q = p;
-		while (*p != '\0' && !isspace((unsigned char) *p))
-			p++;
-		if (*p != '\0') {
-			*p++ = '\0';
-			while (isspace((unsigned char) *p))
-				p++;
-		}
-		interpret_fmt_line(q, p, 0);
-	}
-	fclose(fp);
-	nbfiles--;
-	return 0;
-}
-
 /* -- start a new font -- */
 void set_font(int ft)
 {
@@ -1148,17 +1201,18 @@ void set_font(int ft)
 	f = &cfmt.font_tb[ft];
 	f2 = &cfmt.font_tb[outft];
 	outft = ft;
-	if (f->fnum == f2->fnum && f->size == f2->size)
-		return;
 	fnum = f->fnum;
-	if (!used_font[fnum]) {
+	if (fnum == f2->fnum && f->size == f2->size)
+		return;
+	if (!used_font[fnum]
+	 && epsf != 2 && !svg) {
 		error(1, 0,
-		      "Font \"%s\" not predefined; using first in list",
+		      "Font '%s' not predefined; using first in list",
 		      fontnames[fnum]);
 		fnum = 0;
 	}
 	if (f->size == 0) {
-		error(0, 0, "Font \"%s\" with a null size - set to 8",
+		error(0, 0, "Font '%s' with a null size - set to 8",
 		      fontnames[fnum]);
 		f->size = 8;
 	}
